#include "android.hpp"

#include "tm_ostream.hpp"
#include "boot.hpp"
#include "file.hpp"
#include "tm_timer.hpp"
#include "data_cache.hpp"

#include <QApplication>
#include <QFile>
#include <QTextStream>
#include <QDir>
#include <QFileInfo>
#include <QStringList>
#include <QDebug>
#include <QDirIterator>
#include <QStandardPaths>
#include <QTextBrowser>
#include <QJniObject>

#include <libguile.h>

void guile_log_function(const char *cmsg, int len)
{
    // Convert the c string to a QString.
    // The c string is not null terminated, so we need to specify the length.
    QString msg = QString::fromStdString(std::string(cmsg, len));
    qDebug() << msg;
}

QString getPublicHomePath()
{
    // todo : call getExternalFilesDir
    QStringList paths = QStandardPaths::standardLocations(QStandardPaths::HomeLocation);
    qDebug() << "Home paths: " << paths;
    if (paths.size() == 0)
    {
        return QDir::homePath();
    }
    else
    {
        return paths[0];
    }
}

QMap<QString, QString> *asset_files;

void android_extract_from_asset(QString tmpath)
{
    asset_files = new QMap<QString, QString>();
    qDebug() << "Starting the extraction of TeXmacs files from the assets.";
    qDebug() << "Opening assets:resource_files.txt";
    QFile resource_files("assets:raw/resource_files.txt");
    if (!resource_files.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        qDebug() << "Cannot open resource_files.txt";
        throw std::runtime_error("Cannot open resource_files.txt");
    }
    QTextStream in(&resource_files);
    // read line by line
    while (!in.atEnd())
    {
        QString line = in.readLine();
        if (line == "")
            continue;
        // split the line in two parts
        QStringList parts = line.split(" ");
        if (parts.size() != 2)
            continue;
        // the first part is the id of the resource
        QString id = parts[0];
        // the second part is the name of the file (including the path)
        QString filename = tmpath + "/" + parts[1];
        qDebug() << "Extracting " << id << " to " << filename;
        QFile file("assets:raw/" + id + ".raw");
        if (!file.open(QIODevice::ReadOnly))
            throw std::runtime_error("Cannot open " + id.toStdString() + ".raw");
        QByteArray data = file.readAll();
        file.close();
        // recursively create the directory
        QDir dir;
        dir.mkpath(QFileInfo(filename).path());

        asset_files->insert(filename, data);

        // check that the file does not exist
        if (QFile::exists(filename))
        {
            qDebug() << "File " << filename << " already exists";
            continue;
        }
        
        QFile new_file(filename);
        if (!new_file.open(QIODevice::WriteOnly))
            throw std::runtime_error("Cannot open " + filename.toStdString());
        new_file.write(data);
        new_file.close();
    }
    resource_files.close();
}

QTextBrowser *welcomeScreen;

void init_android()
{
    QString publicHomePath = getPublicHomePath();
    QString homePath = publicHomePath + "/.TeXmacs";
    QString path = publicHomePath + "/TeXmacs";
    QString progsPath = publicHomePath + "/TeXmacs/progs";
    QString pluginsPath = publicHomePath + "/TeXmacs/plugins";

    qputenv("TEXMACS_HOME_PATH", homePath.toUtf8());
    qputenv("TEXMACS_PATH", path.toUtf8());
    qputenv("TEXMACS_PROGS_PATH", progsPath.toUtf8());
    qputenv("TEXMACS_PLUGINS_PATH", pluginsPath.toUtf8());
    qputenv("GUILE_LOAD_PATH", progsPath.toUtf8());

    welcomeScreen = new QTextBrowser;

    QString htmlContent = R"(
        <html>
        <head>
            <style>
                body {
                    background-color: #f2f2f2;
                    display: flex;
                    flex-direction: column;
                    justify-content: center;
                    align-items: center;
                    height: 100vh;
                    font-family: Arial, sans-serif;
                }
                h1 {
                    color: #333;
                    font-size: 36px;
                    margin-bottom: 20px;
                }
                p {
                    color: #666;
                    font-size: 18px;
                }
            </style>
        </head>
        <body>
            <h1>Welcome to TeXmacs</h1>
            <p>Loading...</p>
        </body>
        </html>
    )";

    welcomeScreen->setHtml(htmlContent);
    welcomeScreen->setFixedSize(400, 200); // Adjust the size as needed
    welcomeScreen->setWindowFlags(Qt::WindowStaysOnTopHint);
    welcomeScreen->show();

    QCoreApplication::processEvents();

    qDebug() << "Extracting TeXmacs files from assets";
    android_extract_from_asset(publicHomePath);
    qDebug() << "Setting log function";
    scm_set_log_function(guile_log_function);
}

void hide_splash_screen()
{
    welcomeScreen->hide();
}
